# coding: utf-8
from __future__ import unicode_literals

import re

from .common import InfoExtractor
from ..utils import (
    ExtractorError,
    float_or_none,
)


class VGTVIE(InfoExtractor):
    IE_DESC = 'VGTV and BTTV'
    _VALID_URL = r'''(?x)
                    (?:
                        vgtv:|
                        http://(?:www\.)?
                    )
                    (?P<host>vgtv|bt)
                    (?:
                        :|
                        \.no/(?:tv/)?\#!/(?:video|live)/
                    )
                    (?P<id>[0-9]+)
                    '''
    _TESTS = [
        {
            # streamType: vod
            'url': 'http://www.vgtv.no/#!/video/84196/hevnen-er-soet-episode-10-abu',
            'md5': 'b8be7a234cebb840c0d512c78013e02f',
            'info_dict': {
                'id': '84196',
                'ext': 'mp4',
                'title': 'Hevnen er søt: Episode 10 - Abu',
                'description': 'md5:e25e4badb5f544b04341e14abdc72234',
                'thumbnail': 're:^https?://.*\.jpg',
                'duration': 648.000,
                'timestamp': 1404626400,
                'upload_date': '20140706',
                'view_count': int,
            },
        },
        {
            # streamType: wasLive
            'url': 'http://www.vgtv.no/#!/live/100764/opptak-vgtv-foelger-em-kvalifiseringen',
            'info_dict': {
                'id': '100764',
                'ext': 'flv',
                'title': 'OPPTAK: VGTV følger EM-kvalifiseringen',
                'description': 'md5:3772d9c0dc2dff92a886b60039a7d4d3',
                'thumbnail': 're:^https?://.*\.jpg',
                'duration': 9103.0,
                'timestamp': 1410113864,
                'upload_date': '20140907',
                'view_count': int,
            },
            'params': {
                # m3u8 download
                'skip_download': True,
            },
        },
        {
            # streamType: live
            'url': 'http://www.vgtv.no/#!/live/113063/direkte-v75-fra-solvalla',
            'info_dict': {
                'id': '113063',
                'ext': 'flv',
                'title': 're:^DIREKTE: V75 fra Solvalla [0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}$',
                'description': 'md5:b3743425765355855f88e096acc93231',
                'thumbnail': 're:^https?://.*\.jpg',
                'duration': 0,
                'timestamp': 1432975582,
                'upload_date': '20150530',
                'view_count': int,
            },
            'params': {
                # m3u8 download
                'skip_download': True,
            },
        },
        {
            'url': 'http://www.bt.no/tv/#!/video/100250/norling-dette-er-forskjellen-paa-1-divisjon-og-eliteserien',
            'only_matching': True,
        },
    ]

    def _real_extract(self, url):
        mobj = re.match(self._VALID_URL, url)
        video_id = mobj.group('id')
        host = mobj.group('host')

        HOST_WEBSITES = {
            'vgtv': 'vgtv',
            'bt': 'bttv',
        }

        data = self._download_json(
            'http://svp.vg.no/svp/api/v1/%s/assets/%s?appName=%s-website'
            % (host, video_id, HOST_WEBSITES[host]),
            video_id, 'Downloading media JSON')

        if data.get('status') == 'inactive':
            raise ExtractorError(
                'Video %s is no longer available' % video_id, expected=True)

        streams = data['streamUrls']
        stream_type = data.get('streamType')

        formats = []

        hls_url = streams.get('hls')
        if hls_url:
            formats.extend(self._extract_m3u8_formats(
                hls_url, video_id, 'mp4', m3u8_id='hls'))

        hds_url = streams.get('hds')
        # wasLive hds are always 404
        if hds_url and stream_type != 'wasLive':
            formats.extend(self._extract_f4m_formats(
                hds_url + '?hdcore=3.2.0&plugin=aasp-3.2.0.77.18',
                video_id, f4m_id='hds'))

        mp4_url = streams.get('mp4')
        if mp4_url:
            _url = hls_url or hds_url
            MP4_URL_TEMPLATE = '%s/%%s.%s' % (mp4_url.rpartition('/')[0], mp4_url.rpartition('.')[-1])
            for mp4_format in _url.split(','):
                m = re.search('(?P<width>\d+)_(?P<height>\d+)_(?P<vbr>\d+)', mp4_format)
                if not m:
                    continue
                width = int(m.group('width'))
                height = int(m.group('height'))
                vbr = int(m.group('vbr'))
                formats.append({
                    'url': MP4_URL_TEMPLATE % mp4_format,
                    'format_id': 'mp4-%s' % vbr,
                    'width': width,
                    'height': height,
                    'vbr': vbr,
                    'preference': 1,
                })
        self._sort_formats(formats)

        return {
            'id': video_id,
            'title': self._live_title(data['title']),
            'description': data['description'],
            'thumbnail': data['images']['main'] + '?t[]=900x506q80',
            'timestamp': data['published'],
            'duration': float_or_none(data['duration'], 1000),
            'view_count': data['displays'],
            'formats': formats,
            'is_live': True if stream_type == 'live' else False,
        }


class BTArticleIE(InfoExtractor):
    IE_NAME = 'bt:article'
    IE_DESC = 'Bergens Tidende Articles'
    _VALID_URL = 'http://(?:www\.)?bt\.no/(?:[^/]+/)+(?P<id>[^/]+)-\d+\.html'
    _TEST = {
        'url': 'http://www.bt.no/nyheter/lokalt/Kjemper-for-internatet-1788214.html',
        'md5': 'd055e8ee918ef2844745fcfd1a4175fb',
        'info_dict': {
            'id': '23199',
            'ext': 'mp4',
            'title': 'Alrekstad internat',
            'description': 'md5:dc81a9056c874fedb62fc48a300dac58',
            'thumbnail': 're:^https?://.*\.jpg',
            'duration': 191,
            'timestamp': 1289991323,
            'upload_date': '20101117',
            'view_count': int,
        },
    }

    def _real_extract(self, url):
        webpage = self._download_webpage(url, self._match_id(url))
        video_id = self._search_regex(
            r'SVP\.Player\.load\(\s*(\d+)', webpage, 'video id')
        return self.url_result('vgtv:bt:%s' % video_id, 'VGTV')


class BTVestlendingenIE(InfoExtractor):
    IE_NAME = 'bt:vestlendingen'
    IE_DESC = 'Bergens Tidende - Vestlendingen'
    _VALID_URL = 'http://(?:www\.)?bt\.no/spesial/vestlendingen/#!/(?P<id>\d+)'
    _TEST = {
        'url': 'http://www.bt.no/spesial/vestlendingen/#!/86588',
        'md5': 'd7d17e3337dc80de6d3a540aefbe441b',
        'info_dict': {
            'id': '86588',
            'ext': 'mov',
            'title': 'Otto Wollertsen',
            'description': 'Vestlendingen Otto Fredrik Wollertsen',
            'timestamp': 1430473209,
            'upload_date': '20150501',
        },
    }

    def _real_extract(self, url):
        return self.url_result('xstream:btno:%s' % self._match_id(url), 'Xstream')



# coding: utf-8
from __future__ import unicode_literals

import re

from .common import InfoExtractor
from ..compat import compat_urllib_parse_urlparse
from ..utils import (
    ExtractorError,
    qualities,
    unified_strdate,
    clean_html,
)


class UltimediaIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?ultimedia\.com/default/index/video[^/]+/id/(?P<id>[\d+a-z]+)'
    _TESTS = [{
        # news
        'url': 'https://www.ultimedia.com/default/index/videogeneric/id/s8uk0r',
        'md5': '276a0e49de58c7e85d32b057837952a2',
        'info_dict': {
            'id': 's8uk0r',
            'ext': 'mp4',
            'title': 'Loi sur la fin de vie: le texte prévoit un renforcement des directives anticipées',
            'description': 'md5:3e5c8fd65791487333dda5db8aed32af',
            'thumbnail': 're:^https?://.*\.jpg',
            'upload_date': '20150317',
        },
    }, {
        # music
        'url': 'https://www.ultimedia.com/default/index/videomusic/id/xvpfp8',
        'md5': '2ea3513813cf230605c7e2ffe7eca61c',
        'info_dict': {
            'id': 'xvpfp8',
            'ext': 'mp4',
            'title': "Two - C'est la vie (Clip)",
            'description': 'Two',
            'thumbnail': 're:^https?://.*\.jpg',
            'upload_date': '20150224',
        },
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        webpage = self._download_webpage(url, video_id)

        deliver_url = self._proto_relative_url(self._search_regex(
            r'<iframe[^>]+src="((?:https?:)?//(?:www\.)?ultimedia\.com/deliver/[^"]+)"',
            webpage, 'deliver URL'), compat_urllib_parse_urlparse(url).scheme + ':')

        deliver_page = self._download_webpage(
            deliver_url, video_id, 'Downloading iframe page')

        if '>This video is currently not available' in deliver_page:
            raise ExtractorError(
                'Video %s is currently not available' % video_id, expected=True)

        player = self._parse_json(
            self._search_regex(
                r"jwplayer\('player(?:_temp)?'\)\.setup\(({.+?})\)\.on",
                deliver_page, 'player'),
            video_id)

        quality = qualities(['flash', 'html5'])
        formats = []
        for mode in player['modes']:
            video_url = mode.get('config', {}).get('file')
            if not video_url:
                continue
            if re.match(r'https?://www\.youtube\.com/.+?', video_url):
                return self.url_result(video_url, 'Youtube')
            formats.append({
                'url': video_url,
                'format_id': mode.get('type'),
                'quality': quality(mode.get('type')),
            })
        self._sort_formats(formats)

        thumbnail = player.get('image')

        title = clean_html((
            self._html_search_regex(
                r'(?s)<div\s+id="catArticle">.+?</div>(.+?)</h1>',
                webpage, 'title', default=None) or
            self._search_regex(
                r"var\s+nameVideo\s*=\s*'([^']+)'",
                deliver_page, 'title')))

        description = clean_html(self._html_search_regex(
            r'(?s)<span>Description</span>(.+?)</p>', webpage,
            'description', fatal=False))

        upload_date = unified_strdate(self._search_regex(
            r'Ajouté le\s*<span>([^<]+)', webpage,
            'upload date', fatal=False))

        return {
            'id': video_id,
            'title': title,
            'description': description,
            'thumbnail': thumbnail,
            'upload_date': upload_date,
            'formats': formats,
        }



# coding: utf-8
from __future__ import unicode_literals

import re

from .common import InfoExtractor
from ..compat import compat_urlparse


class BaiduVideoIE(InfoExtractor):
    IE_DESC = '百度视频'
    _VALID_URL = r'http://v\.baidu\.com/(?P<type>[a-z]+)/(?P<id>\d+)\.htm'
    _TESTS = [{
        'url': 'http://v.baidu.com/comic/1069.htm?frp=bdbrand&q=%E4%B8%AD%E5%8D%8E%E5%B0%8F%E5%BD%93%E5%AE%B6',
        'info_dict': {
            'id': '1069',
            'title': '中华小当家 TV版 (全52集)',
            'description': 'md5:395a419e41215e531c857bb037bbaf80',
        },
        'playlist_count': 52,
    }, {
        'url': 'http://v.baidu.com/show/11595.htm?frp=bdbrand',
        'info_dict': {
            'id': '11595',
            'title': 're:^奔跑吧兄弟',
            'description': 'md5:1bf88bad6d850930f542d51547c089b8',
        },
        'playlist_mincount': 3,
    }]

    def _real_extract(self, url):
        mobj = re.match(self._VALID_URL, url)
        playlist_id = mobj.group('id')
        category = category2 = mobj.group('type')
        if category == 'show':
            category2 = 'tvshow'

        webpage = self._download_webpage(url, playlist_id)

        playlist_title = self._html_search_regex(
            r'title\s*:\s*(["\'])(?P<title>[^\']+)\1', webpage,
            'playlist title', group='title')
        playlist_description = self._html_search_regex(
            r'<input[^>]+class="j-data-intro"[^>]+value="([^"]+)"/>', webpage,
            playlist_id, 'playlist description')

        site = self._html_search_regex(
            r'filterSite\s*:\s*["\']([^"]*)["\']', webpage,
            'primary provider site')
        api_result = self._download_json(
            'http://v.baidu.com/%s_intro/?dtype=%sPlayUrl&id=%s&site=%s' % (
                category, category2, playlist_id, site),
            playlist_id, 'Get playlist links')

        entries = []
        for episode in api_result[0]['episodes']:
            episode_id = '%s_%s' % (playlist_id, episode['episode'])

            redirect_page = self._download_webpage(
                compat_urlparse.urljoin(url, episode['url']), episode_id,
                note='Download Baidu redirect page')
            real_url = self._html_search_regex(
                r'location\.replace\("([^"]+)"\)', redirect_page, 'real URL')

            entries.append(self.url_result(
                real_url, video_title=episode['single_title']))

        return self.playlist_result(
            entries, playlist_id, playlist_title, playlist_description)
